/* -*-C++-*-
 * $Id: tclMatrix.h,v 1.15 1995/12/15 18:48:16 furnish Exp $
 * $Log: tclMatrix.h,v $
 * Revision 1.15  1995/12/15  18:48:16  furnish
 * Improvements to the C++ Tcl matrix wrapper objects.
 *
 * Revision 1.14  1995/06/09  19:19:32  furnish
 * Corrected CPP logic for handling C++ exceptions on HP.
 *
 * Revision 1.13  1995/06/05  21:44:44  furnish
 * Correct several points of bogosity in the C++ Tcl Matrix shadow class
 * for float's, and add a new one int's.
 *
 * Revision 1.12  1995/06/02  20:30:37  mjl
 * Added missing include of iostream.h for C++ codes including this.
 *
 * Revision 1.11  1995/06/01  21:19:29  mjl
 * Eliminated extraneous include of tcl.h.
 *
 * Revision 1.10  1995/03/21  19:40:32  mjl
 * Added interpreter to tclMatrix struct for more robust resource recovery.
 *
 * Revision 1.9  1994/08/11  01:43:07  furnish
 * Implement redim methods for the C++ shadow class.  Somebody still
 * needs to do it in the Tcl code :-).
 *
 * Revision 1.8  1994/08/05  17:45:31  furnish
 * More on the C++ shadow class.
 *
 * Revision 1.7  1994/06/30  18:38:49  mjl
 * Changed typedef for Mat_int back to an int, and eliminated M?D macros in
 * favor of directly addressing the array.
 *
 * Revision 1.6  1994/06/30  05:45:21  furnish
 * Cobbled together an extension facility which allows a user to define
 * their own subcommands for tclMatricies.  The idea is that you can use
 * this to implement your own matrix processing commands entirely on the
 * compiled side.  fft's, matrix multiplication, inversion, etc.
 *
 * Revision 1.5  1994/06/26  05:16:15  furnish
 * Implemented C++ wrapper class for tclMatrix, enabling easy use of a
 * tclMatrix from compiled extension commands.  Needs to have sibling
 * classes created by someone who cares.
 *
 * Revision 1.4  1994/06/25  20:35:49  mjl
 * Changed typedef for Mat_int to long.  Maybe I should have a Mat_long
 * instead?  Added put/get function handlers to matrix struct (determined
 * when created, dependent on type).
 *
 * Revision 1.3  1994/06/24  20:37:55  mjl
 * Changed name of struct to tclMatrix to avoid conflicts with C++ Matrix
 * classes.  Put in ifdef-ed extern "C" linkage statements for C++.
 *
 * Revision 1.2  1994/06/16  21:57:11  mjl
 * Added matrix operator name to command data struct.
 *
 * Revision 1.1  1994/06/16  19:49:05  mjl
 * Header file for new Tcl matrix command.
 */

/*
    tclMatrix.h

    Copyright 1994, 1995
    Maurice LeBrun			mjl@dino.ph.utexas.edu
    Institute for Fusion Studies	University of Texas at Austin

    This library is free software; you can redistribute it and/or
    modify it under the terms of the GNU Library General Public
    License as published by the Free Software Foundation; either
    version 2 of the License, or (at your option) any later version.

    This library is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
    Library General Public License for more details.

    You should have received a copy of the GNU Library General Public
    License along with this library; if not, write to the Free
    Software Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.

* * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * *
	
    Contains declarations for Tcl "Matrix" command.
    C functions that need access to the matrix data will need
    to include this file.
*/

#ifndef __TCLMATRIX_H__
#define __TCLMATRIX_H__

#ifdef __cplusplus
extern "C" {
#endif
    
#include <stdlib.h>
#include <tcl.h>
#include "plConfig.h"

#ifdef PL_USE_TCL_TK_MALLOC
	#include <tcl.h>
	#define malloc ckalloc
	#define free ckfree
	#ifndef free_mem
	#define free_mem(a) \
	    if (a != NULL) { free((char *) a); a = NULL; }
	#endif
#endif
#ifdef PL_USE_TCL_STDIO
	#define TCLSTDIO
	#define REDEFINESTDIO
	#include "tclstdio.h"
#endif
#ifndef free_mem
/* Used to help ensure everything malloc'ed gets freed */
#define free_mem(a) \
    if (a != NULL) { free((void *) a); a = NULL; }
#endif
#if defined(DOUBLE) || defined(PL_DOUBLE)
typedef double Mat_float;
#else
typedef float  Mat_float;
#endif

#if defined(MSDOS) || defined(DOUBLE) || defined(PL_DOUBLE)
typedef long  Mat_int;
#else
typedef int   Mat_int;
#endif

int Matrix_Init (Tcl_Interp*);

enum { TYPE_FLOAT, TYPE_INT };

/* Arrays are column dominant (normal C ordering) */
/* Array elements are stored contiguously */
/* Require dimension <= 3 for simplicity */

#define MAX_ARRAY_DIM 3

/* Useful macros for index calculations */

#define I3D(i,j,k)	k + matPtr->n[2] * (I2D(i,j))
#define I2D(i,j)	j + matPtr->n[1] * (I1D(i))
#define I1D(i)		i

/* Matrix operator data */

typedef struct {
    int type;			/* Data type */
    int len;			/* Total length of array */
    int dim;			/* Number of dimensions */
    int n[MAX_ARRAY_DIM];	/* Holds array length in each dimension */
    int tracing;		/* Set if not persistent */

    char *name;			/* Matrix operator name, malloc'ed */

    Mat_float *fdata;		/* Floating point data, malloc'ed */
    Mat_int   *idata;		/* Integer data, malloc'ed */

    Tcl_Interp *interp;		/* Interpreter where command is installed */

/* These do the put/get operations for each supported type */
    void (*put) (ClientData clientData, int index, char *string);
    void (*get) (ClientData clientData, int index, char *string);
} tclMatrix;

/* Function prototypes */

/* This procedure is invoked to process the "matrix" Tcl command. */

int
Tcl_MatrixCmd(ClientData clientData, Tcl_Interp *interp,
	      int argc, char **argv);

/* Returns a pointer to the specified matrix operator's data. */

tclMatrix *
Tcl_GetMatrixPtr(Tcl_Interp *interp, const char *matName);

/* Some stuff for handling extension subcommands. */

typedef int (*tclMatrixXtnsnProc) ( tclMatrix *pm, Tcl_Interp *interp,
				    int argc, char *argv[] );

typedef struct tclMatrixXtnsnDescr {
    char *cmd;
    tclMatrixXtnsnProc cmdproc;
    struct tclMatrixXtnsnDescr *next;
} tclMatrixXtnsnDescr;

int Tcl_MatrixInstallXtnsn( char *cmd, tclMatrixXtnsnProc proc );


typedef tclMatrix * (*Tcl_GetMatrixPtrProc)(Tcl_Interp *, const char *);
void Tcl_RegisterMatrixType(Tcl_GetMatrixPtrProc p);
tclMatrix * Tcl_GetMatrixPtr(Tcl_Interp *interp, const char *matName);

#ifdef __cplusplus
}
#ifdef PLPLOT_MATRIX_CLASSES
#include "tclMatrixClasses.h"
#endif
#endif



#endif	/* __TCLMATRIX_H__ */
