#include "tclMatrix.h"

/*---------------------------------------------------------------------------//
// Since C++ does not generally have a per-platform ABI the way C
// does, we stick to a totally inline class declaration and
// definition.  That way you don't have to keep a separate version of
// libplplot*.a for each compiler you'd like to use.

// Start by setting up some important macros.

// HP C++ has exceptions, but most other compilers don't.  Will embellish
// this as necessary.
*/

#include <iostream.h>

#ifdef throw
#define TCL_NO_UNDEF
#endif

#ifndef throw
#ifdef __hpux
#if defined(__GNUC__) || defined(__lucid) || defined(__CENTERLINE__) \
|| defined(CENTERLINE_CLPP)
#define NO_XCPT
#endif
#else
#define NO_XCPT
#endif

#ifdef NO_XCPT
#define try
#define throw(a) \
{ cerr << "THROW: " << #a << " from " << __FILE__ \
       << " line " << __LINE__ << endl << flush; abort(); }
#define catch(a) if (0)
#define Throw
#else
#define Throw throw
#endif
#endif

#define tMat_Assert(a,b) if (!(a)) \
{ cerr << "Assertion " << #a << " failed in " << __FILE__ \
       << " at line " << __LINE__ << endl << flush; \
  throw(b); }

/*---------------------------------------------------------------------------//
// class TclMatFloat

// This class provides a convenient way to access the data of a
// tclMatrix from within compiled code.  Someone should make clones of
// this class for the other tclMatrix supported data types.
//---------------------------------------------------------------------------*/

class TclMatFloat {
  public:
    tclMatrix *matPtr;
  public:	
  	operator tclMatrix* () { return matPtr; }

	operator Mat_float* () {
		return matPtr->fdata;
	}
	/* Warning: very unsafe function */
    TclMatFloat(void)
	: matPtr(0)
    {
    }
    
	
    TclMatFloat( tclMatrix *ptm)
	: matPtr(ptm)
    {
	tMat_Assert( matPtr->type == TYPE_FLOAT, "Type mismatch" );
    }

    int Dimensions() const { return matPtr->dim; }

    int dim_size( int d ) const {
	tMat_Assert( d < matPtr->dim, "Range error." );
	return matPtr->n[d];
    }

    void redim( int nx )
    {
	free_mem( matPtr->fdata );
	matPtr->dim = 1;
	matPtr->n[0] = nx;
	matPtr->len = nx;
	matPtr->fdata = (Mat_float *) malloc( matPtr->len *
					      sizeof(Mat_float) );
    }

    void redim( int nx, int ny )
    {
	free_mem( matPtr->fdata );
	matPtr->dim = 2;
	matPtr->n[0] = nx;
	matPtr->n[1] = ny;
	matPtr->len = nx * ny;
	matPtr->fdata = (Mat_float *) malloc( matPtr->len * 
					      sizeof(Mat_float) );
    }

    void redim( int nx, int ny, int nz )
    {
	free_mem( matPtr->fdata );
	matPtr->dim = 3;
	matPtr->n[0] = nx;
	matPtr->n[1] = ny;
	matPtr->n[2] = nz;
	matPtr->len = nx * ny * nz;
	matPtr->fdata = (Mat_float *) malloc( matPtr->len * 
					      sizeof(Mat_float) );
    }

    Mat_float& operator()( int i )
    {
	tMat_Assert( matPtr->dim == 1, "Wrong number of indices." );
	tMat_Assert( i >= 0 && i < matPtr->n[0],
		     "Out of bounds reference" );

	return matPtr->fdata[i];
    }

    Mat_float& operator()( int i, int j )
    {
	tMat_Assert( matPtr->dim == 2, "Wrong number of indices." );
	tMat_Assert( i >= 0 && i < matPtr->n[0] &&
		     j >= 0 && j < matPtr->n[1],
		     "Out of bounds reference" );
		
	return matPtr->fdata[I2D(i,j)];
    }

    Mat_float& operator()( int i, int j, int k )
    {
	tMat_Assert( matPtr->dim = 3, "Wrong number of indices." );
	tMat_Assert( i >= 0 && i < matPtr->n[0] &&
		     j >= 0 && j < matPtr->n[1] &&
		     k >= 0 && k < matPtr->n[2],
		     "Out of bounds reference" );

	return matPtr->fdata[I3D(i,j,k)];
    }
};

/*---------------------------------------------------------------------------//
// class TclMatInt

// This class provides a convenient way to access the data of a
// tclMatrix from within compiled code.  This is just like TclMatFloat above,
// but for ints.
//---------------------------------------------------------------------------*/

class TclMatInt {
  public:
    tclMatrix *matPtr;
  public:
  	operator tclMatrix* () { return matPtr; }
	operator Mat_int* () {
		return matPtr->idata;
	}
	/* warning very unsafe function */
	TclMatInt(void): matPtr(0) {
	}
	
    TclMatInt( tclMatrix *ptm)
	: matPtr(ptm)
    {
	tMat_Assert( matPtr->type == TYPE_INT, "Type mismatch" );
    }

    int Dimensions() const { return matPtr->dim; }

    int dim_size( int d ) const {
	tMat_Assert( d < matPtr->dim, "Range error." );
	return matPtr->n[d];
    }

    void redim( int nx )
    {
	free_mem( matPtr->idata );
	matPtr->dim = 1;
	matPtr->n[0] = nx;
	matPtr->len = nx;
	matPtr->idata = (Mat_int *) malloc( matPtr->len * sizeof(Mat_int) );
    }

    void redim( int nx, int ny )
    {
	free_mem( matPtr->idata );
	matPtr->dim = 2;
	matPtr->n[0] = nx;
	matPtr->n[1] = ny;
	matPtr->len = nx * ny;
	matPtr->idata = (Mat_int *) malloc( matPtr->len * sizeof(Mat_int) );
    }

    void redim( int nx, int ny, int nz )
    {
	free_mem( matPtr->idata );
	matPtr->dim = 3;
	matPtr->n[0] = nx;
	matPtr->n[1] = ny;
	matPtr->n[2] = nz;
	matPtr->len = nx * ny * nz;
	matPtr->idata = (Mat_int *) malloc( matPtr->len * sizeof(Mat_int) );
    }

    Mat_int& operator()( int i )
    {
	tMat_Assert( matPtr->dim == 1, "Wrong number of indices." );
	tMat_Assert( i >= 0 && i < matPtr->n[0],
		     "Out of bounds reference" );

	return matPtr->idata[i];
    }

    Mat_int& operator()( int i, int j )
    {
	tMat_Assert( matPtr->dim == 2, "Wrong number of indices." );
	tMat_Assert( i >= 0 && i < matPtr->n[0] &&
		     j >= 0 && j < matPtr->n[1],
		     "Out of bounds reference" );
		
	return matPtr->idata[I2D(i,j)];
    }

    Mat_int& operator()( int i, int j, int k )
    {
	tMat_Assert( matPtr->dim = 3, "Wrong number of indices." );
	tMat_Assert( i >= 0 && i < matPtr->n[0] &&
		     j >= 0 && j < matPtr->n[1] &&
		     k >= 0 && k < matPtr->n[2],
		     "Out of bounds reference" );

	return matPtr->idata[I3D(i,j,k)];
    }
};

struct TclMatInt2: public TclMatInt {
	TclMatInt2(void):TclMatInt(){}
};

struct TclMatFloat2: public TclMatFloat {
	TclMatFloat2(void):TclMatFloat(){}
};

#ifndef TCL_NO_UNDEF

#ifdef NO_XCPT
#undef NO_XCPT
#undef try
#undef throw
#undef Throw
#undef catch
#endif

#endif

#undef tMat_Assert
